@@ -46,6 +46,8 @@ enum {
 	LAST_SIGNAL
 };
 
+static GtkSourceTagStyle normal_style = { TRUE, 0 };
+
 static guint 	 signals[LAST_SIGNAL] = { 0 };
 
 static GObjectClass 	 *parent_class  = NULL;
@@ -180,8 +182,6 @@ gtk_source_language_finalize (GObject *object)
 		
 	if (lang->priv != NULL)
 	{
-		g_print ("Finalize lang: %s\n", lang->priv->name);
-
 		g_free (lang->priv->lang_file_name);
 
 		xmlFree (lang->priv->translation_domain);
@@ -214,10 +214,10 @@ static const gchar invalid_chars[] = " \t\r\n\"$&<>,+=#!()'|{}[]?~`;%\\";
 
 /**
  * escape_id:
- * @arbitrary_text: some text in any encoding or format
- * @len: length of @arbitrary_text in bytes, or -1 if @arbitrary_text is nul-terminated
+ * @arbitrary_text: some text in any encoding or format.
+ * @len: length of @arbitrary_text in bytes, or -1 if @arbitrary_text is nul-terminated.
  * 
- * Return value: a nul-terminated valid part of a GConf key
+ * Return value: a null-terminated valid part of a GConf key.
  **/
 static gchar*
 escape_id (const gchar *arbitrary_text, gint len)
@@ -421,7 +421,7 @@ gtk_source_language_get_name (GtkSourceLanguage *language)
  *
  * Returns the localized section of the language.
  * Each language belong to a section (ex. HTML belogs to the
- * Markup section)
+ * Markup section).
  *
  * Return value: the section of @language.
  **/
@@ -434,6 +434,15 @@ gtk_source_language_get_section	(GtkSourceLanguage *language)
 	return g_strdup (language->priv->section);
 }
 
+/**
+ * gtk_source_language_get_mime_types:
+ * @language: a #GtkSourceLanguage.
+ *
+ * Returns a list of mime types for the given @language.  After usage you should
+ * free each element of the list as well as the list itself.
+ *
+ * Return value: a list of mime types (strings). 
+ **/
 GSList *
 gtk_source_language_get_mime_types (GtkSourceLanguage *language)
 {
@@ -544,6 +553,15 @@ get_mime_types_from_file (GtkSourceLanguage *language)
 	return mime_types;
 }
 
+/**
+ * gtk_source_language_set_mime_types:
+ * @language: a #GtkSourceLanguage
+ * @mime_types: a list of mime types (strings).
+ *
+ * Sets a list of @mime_types for the given @language.
+ * If @mime_types is %NULL this function will use the default mime
+ * types from the language file.
+ **/
 void 
 gtk_source_language_set_mime_types (GtkSourceLanguage	*language,
 				    const GSList	*mime_types)
@@ -1178,6 +1196,15 @@ language_file_parse (GtkSourceLanguage *language,
 	return tag_list;
 }
 
+/**
+ * gtk_source_language_get_tags:
+ * @language: a #GtkSourceLanguage.
+ *
+ * Returns a list of tags for the given @language.  You should unref the tags
+ * and free the list after usage.
+ *
+ * Return value: a list of #GtkSourceTag objects.
+ **/
 GSList *
 gtk_source_language_get_tags (GtkSourceLanguage *language)
 {
@@ -1229,6 +1256,16 @@ gtk_source_language_lazy_init_hash_tables (GtkSourceLanguage *language)
 	return TRUE;
 }
 
+/**
+ * gtk_source_language_get_tag_style:
+ * @language: a #GtkSourceLanguage.
+ * @tag_id: the ID of a #GtkSourceTag.
+ *
+ * Gets the style of the tag whose ID is @tag_id. If the style is 
+ * not defined then returns the default style.
+ *
+ * Return value: a #GtkSourceTagStyle.
+ **/
 GtkSourceTagStyle *
 gtk_source_language_get_tag_style (GtkSourceLanguage *language, 
 				   const gchar       *tag_id)
@@ -1254,6 +1291,15 @@ gtk_source_language_get_tag_style (GtkSourceLanguage *language,
 	}
 }
 
+/**
+ * gtk_source_language_get_tag_default_style:
+ * @language: a #GtkSourceLanguage.
+ * @tag_id: the ID of a #GtkSourceTag.
+ *
+ * Gets the default style of the tag whose ID is @tag_id. 
+ *
+ * Return value: a #GtkSourceTagStyle.
+ **/
 GtkSourceTagStyle*
 gtk_source_language_get_tag_default_style (GtkSourceLanguage *language,
 					   const gchar       *tag_id)
@@ -1271,39 +1317,56 @@ gtk_source_language_get_tag_default_style (GtkSourceLanguage *language,
 
 	if (style_name != NULL)
 	{
-		const GtkSourceTagStyle *tmp;
+		GtkSourceTagStyle *tmp;
 
 		g_return_val_if_fail (language->priv->style_scheme != NULL, NULL);
 
 		tmp = gtk_source_style_scheme_get_tag_style (language->priv->style_scheme,
 							     style_name);
 		if (tmp == NULL)
-			return NULL;
+			return gtk_source_tag_style_copy (&normal_style);
 		else
-			return gtk_source_tag_style_copy (tmp);
+			return tmp;
 	}
 	else
-		return NULL;
+		return gtk_source_tag_style_copy (&normal_style);
 }
 
+/**
+ * gtk_source_language_set_tag_style:
+ * @language: a #GtkSourceLanguage.
+ * @tag_id: the ID of a #GtkSourceTag.
+ * @style: a #GtkSourceTagStyle.
+ *
+ * Sets the @style of the tag whose ID is @tag_id. If @style is %NULL
+ * restore the default style.
+ **/
 void 
 gtk_source_language_set_tag_style (GtkSourceLanguage       *language,
 				   const gchar             *tag_id,
 				   const GtkSourceTagStyle *style)
 {
-	GtkSourceTagStyle *ts;
-
 	g_return_if_fail (GTK_SOURCE_LANGUAGE (language));
 	g_return_if_fail (tag_id != NULL);
 
 	if (!gtk_source_language_lazy_init_hash_tables (language))
 			return;	
 	
-	ts = gtk_source_tag_style_copy (style);
+	if (style != NULL)
+	{
+		GtkSourceTagStyle *ts;
+		
+		ts = gtk_source_tag_style_copy (style);
 
-	g_hash_table_insert (language->priv->tag_id_to_style,
-			     g_strdup (tag_id),
-			     ts);
+		g_hash_table_insert (language->priv->tag_id_to_style,
+				     g_strdup (tag_id),
+				     ts);
+	}
+	else
+	{
+		g_hash_table_remove (language->priv->tag_id_to_style,
+				     tag_id);
+	}
 
 	g_signal_emit (G_OBJECT (language),
 		       signals[TAG_STYLE_CHANGED], 
@@ -1311,6 +1374,14 @@ gtk_source_language_set_tag_style (GtkSourceLanguage       *language,
 		       tag_id);
 }
 
+/**
+ * gtk_source_language_get_style_scheme:
+ * @language: a #GtkSourceLanguage.
+ * 
+ * Gets the style scheme associated with the given @language.
+ *
+ * Return value: a #GtkSourceStyleScheme.
+ **/
 GtkSourceStyleScheme *
 gtk_source_language_get_style_scheme (GtkSourceLanguage *language)
 {
@@ -1335,6 +1406,26 @@ emit_tag_style_changed_signal (gpointer  key,
 	return TRUE;
 }
 
+static void
+style_changed_cb (GtkSourceStyleScheme *scheme,
+		  const gchar          *tag_id,
+		  gpointer              user_data)
+{
+	GtkSourceLanguage *language = GTK_SOURCE_LANGUAGE (user_data);
+
+	g_signal_emit (G_OBJECT (language),
+		       signals[TAG_STYLE_CHANGED], 
+		       0, 
+		       tag_id);	
+}
+
+/**
+ * gtk_source_language_set_style_scheme:
+ * @language: a #GtkSourceLanguage.
+ * @scheme: a #GtkSourceStyleScheme.
+ * 
+ * Sets the style scheme of the given @language.
+ **/
 void 
 gtk_source_language_set_style_scheme (GtkSourceLanguage    *language,
 				      GtkSourceStyleScheme *scheme)
@@ -1357,8 +1448,19 @@ gtk_source_language_set_style_scheme (GtkSourceLanguage    *language,
 	g_hash_table_foreach (language->priv->tag_id_to_style_name,
 			      (GHFunc) emit_tag_style_changed_signal,
 			      (gpointer) language);
+
+	g_signal_connect (G_OBJECT (scheme), "style_changed",
+			  G_CALLBACK (style_changed_cb), language);
 }
 
+/**
+ * gtk_source_language_get_escape_char:
+ * @language: a #GtkSourceLanguage.
+ * 
+ * Gets the value of the ESC character in the given @language.
+ * 
+ * Return value: the value of the ESC character.
+ **/
 gunichar 
 gtk_source_language_get_escape_char (GtkSourceLanguage *language)
 {
